package com.unis.hlht.server.redisstream;

import com.unis.hlht.server.server.message.MessageServer;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Range;
import org.springframework.data.redis.connection.stream.Consumer;
import org.springframework.data.redis.connection.stream.MapRecord;
import org.springframework.data.redis.connection.stream.PendingMessages;
import org.springframework.data.redis.connection.stream.RecordId;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.StreamOperations;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.time.Duration;
import java.util.List;
import java.util.Map;

@Slf4j
@Component
public class ScheduleJob {
    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    @Autowired
    private MessageServer messageServer;

    /**
     * 每隔5秒钟，扫描一下有没有等待自己消费的
     * 主要消费那些转组过来的消息，如果转组次数大于1，则进行尝试消费
     */
    @Scheduled(cron = "0/10 * * * * ?")
    public void reportCurrentTime() {
        StreamOperations<String, String, Object> streamOperations = this.redisTemplate.opsForStream();
        /*从消费者的pending队列中读取消息，能够进到这里面的，一定是非业务异常，例如接口超时、服务器宕机等。
        对于业务异常，例如字段解析失败等，丢进异常表或者redis*/
        PendingMessages pendingMessages = streamOperations.pending(BasicAckStreamConsumeListener.NOTICE_UPLOAD_STREAM_KEY, Consumer.from(BasicAckStreamConsumeListener.GROUP, BasicAckStreamConsumeListener.NOTICE_UPLOAD_CONSUMER));
        if (pendingMessages.size() > 0) {
            pendingMessages.forEach(pendingMessage -> {
                // 最后一次消费到现在的间隔
                Duration elapsedTimeSinceLastDelivery = pendingMessage.getElapsedTimeSinceLastDelivery();
                // 转组次数
                long totalDeliveryCount = pendingMessage.getTotalDeliveryCount();
                // 只消费转组次数大于1次的
                if (elapsedTimeSinceLastDelivery.getSeconds() > (60 * 60)) {
                    MapRecord<String, String, Object> entries = null;
                    try {
                        RecordId id = pendingMessage.getId();
                        List<MapRecord<String, String, Object>> result = streamOperations.range(BasicAckStreamConsumeListener.NOTICE_UPLOAD_STREAM_KEY, Range.rightOpen(id.toString(), id.toString()));
                        entries = result.get(0);
                        // 消费消息
                        Map<String, Object> map = entries.getValue();
                        log.info("获取第一次未能成功消费的消息并再次消费（据第一次消费一小时），" +
                                "该消息id={}, 消息value={}, 消费者={}", entries.getId(), entries.getValue(), BasicAckStreamConsumeListener.NOTICE_UPLOAD_CONSUMER);
                        messageServer.deal(map.get("busId") == null ? null : map.get("busId").toString().replaceAll("\"", ""),
                                map.get("tableName").toString().replaceAll("\"", ""),
                                map.get("dwdm") == null ? null : map.get("dwdm").toString().replaceAll("\"", ""));
                    } catch (Exception e) {
                        // 异常处理
                        log.error("第二次消息处理异常", e);
                    } finally {
                        // 手动ack消息
                        if (entries != null) {
                            log.error("两次消费后不看结果，手动ack");
                            streamOperations.acknowledge(BasicAckStreamConsumeListener.GROUP, entries);
                        }
                    }
                }
            });
        }
    }

    @Scheduled(cron = "0 0 4 * * ?")
    public void dealUpload() {
        try {
            messageServer.dealAllApi();
        } catch (Exception e) {
            log.error("凌晨4点上传数据至国家局错误", e);
        }
    }

}